"use strict";

var _interopRequireWildcard = function (obj) { return obj && obj.__esModule ? obj : { "default": obj }; };

exports.JSXElement = JSXElement;
exports.__esModule = true;

var react = _interopRequireWildcard(require("../../helpers/react"));

var t = _interopRequireWildcard(require("../../../types"));

var optional = true;

var immutabilityVisitor = {
  enter: function enter(node, parent, scope, state) {
    if (this.isJSXIdentifier() && react.isCompatTag(node.name)) {
      return;
    }

    if (this.isJSXIdentifier() || this.isIdentifier()) {
      // direct references that we need to track to hoist this to the highest scope we can
      if (this.isReferenced()) {
        console.log(scope.block.type, node.name);
        state.identifiers[node.name] = scope.getBindingIdentifier(node.name);
        return;
      }
    }

    if (this.isJSXClosingElement()) {
      this.skip();
      return;
    }

    // ignore
    if (this.isIdentifier() || this.isJSXMemberExpression()) {
      return;
    }

    state.isImmutable = this.isImmutable();
    if (!state.isImmutable) this.stop();
  }
};

function scopeIsCompatible(scope, state) {
  for (var name in state.identifiers) {
    if (!scope.bindingIdentifierEquals(name, state.identifiers[name])) {
      return false;
    }
  }
  return true;
}

function getLastCompatibleScope(scope, state, lastCompatibleScope) {
  if (scopeIsCompatible(scope, state)) {
    if (scope.path.isProgram()) {
      // we've reached the top, this is pretty pointless though since we can't use `getStatementParent`
      // on this so let's just use the last scope
      return lastCompatibleScope;
    } else {
      // up we go!
      return getLastCompatibleScope(scope.parent, state, scope);
    }
  } else {
    // this scope isn't compatible so just return the last good one
    return lastCompatibleScope;
  }
}

function JSXElement(node, parent, scope, file) {
  // todo - check for `ref` attribute

  var state = {
    identifiers: {},
    isImmutable: true
  };

  this.traverse(immutabilityVisitor, state);
  this.skip();

  if (!state.isImmutable) return;

  var lastCompatibleScope = getLastCompatibleScope(scope, state);

  console.log(state);

  // same scope, can't hoist it up
  if (lastCompatibleScope === scope) return;

  var statementParent = lastCompatibleScope.path.getStatementParent();

  var uid = scope.generateUidIdentifier("ref");
  statementParent.insertBefore([t.variableDeclaration("var", [t.variableDeclarator(uid, node)])]);
  return uid;
}